Package: pcreg
Title: Advanced Methods for Principal Component Analysis and Principal
        Component Regression
Type: Package
Version: 0.1.1
Authors@R: c(person("Dr. Pramit", "Pandit", role = c("aut", "cre"), email = "pramitpandit@gmail.com"),
  person("Dr. Halagundegowda", "G R", role = "aut"),
  person("Dr. Kamidi", "Rahul", role = "aut"),
  person("Dr. S. Gandhi", "Doss", role = "aut"))
Author: Dr. Pramit Pandit [aut, cre],
  Dr. Halagundegowda G R [aut],
  Dr. Kamidi Rahul [aut],
  Dr. S. Gandhi Doss [aut]
Description: Provides a unified framework for principal component analysis (PCA) and principal component regression (PCR), including standard PCA, sparse PCA, robust PCA, and supervised PCA. The package supports automatic selection of the number of components using cumulative variance and elbow methods and integrates PCA with regression modelling through PCR models. It includes tools for PCA suitability assessment using Bartlett's test of sphericity and the Kaiser-Meyer-Olkin (KMO) measure. Visualisation utilities such as scree plots and biplots are provided for interpretation. The methods are designed to handle multicollinearity, outliers, and high-dimensional data, making them suitable for applied statistical modelling and data analysis. The methodology is based on established approaches described in Jolliffe (2002) <doi:10.1007/b98835>, Zou et al. (2006) <doi:10.1111/j.1467-9868.2005.00503.x>, and Hubert et al. (2005) <doi:10.1198/004017004000000563>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, ggplot2, ggrepel, gridExtra, scales, psych, elasticnet,
        robustbase, grid
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-31 09:23:30 UTC; prami
Maintainer: Dr. Pramit Pandit <pramitpandit@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-31 15:00:08 UTC
Built: R 4.5.2; ; 2026-05-31 18:30:55 UTC; unix

# Generated by roxygen2: do not edit by hand

export(pca)
export(pcreg)
export(robust_pca)
export(robust_pcreg)
export(sparse_pca)
export(sparse_pcreg)
export(supervised_pca)
export(supervised_pcreg)
importFrom(elasticnet,spca)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggrepel,geom_text_repel)
importFrom(grid,arrow)
importFrom(grid,unit)
importFrom(gridExtra,grid.arrange)
importFrom(psych,KMO)
importFrom(psych,cortest.bartlett)
importFrom(robustbase,covMcd)
importFrom(scales,percent_format)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,prcomp)
importFrom(stats,sd)
importFrom(stats,step)

#' @title Principal Component Analysis (Custom Scree Style)
#'
#' @description
#' Performs PCA with Bartlett test, KMO measure,
#' and visualizes scree plot and biplot (custom style).
#'
#' @param data A data frame or matrix
#' @param scale. Logical; scale variables
#' @param center Logical; center variables
#' @param verbose Logical; if TRUE, prints Bartlett and KMO results.
#'
#' @return List with Bartlett test, KMO, loadings, scores, variance
#'
#' @importFrom stats cor prcomp complete.cases
#' @importFrom psych cortest.bartlett KMO
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point geom_text
#' @importFrom ggplot2 scale_y_continuous labs theme_minimal theme
#' @importFrom ggplot2 element_text element_line element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#' @export
#' @examples {
#' data(mtcars)
#' Result <- pca(mtcars)
#' }

pca <- function(data, scale. = TRUE, center = TRUE, verbose = FALSE){

  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)
  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  if (ncol(df) < 2) {
    stop("Need at least 2 numeric variables")
  }

  df <- df[stats::complete.cases(df), , drop = FALSE]

  if (nrow(df) < 2) {
    stop("Not enough complete observations")
  }

  cm <- stats::cor(df)

  if (verbose) {
    message("--- PCA Suitability ---")
  }

  singular <- tryCatch(base::det(cm) < 1e-10, error = function(e) TRUE)

  bart <- NULL
  if (!singular) {
    bart <- tryCatch(
      psych::cortest.bartlett(cm, n = nrow(df)),
      error = function(e) NULL
    )
  }

  if (verbose) {
    if (!is.null(bart)) {
      message("Bartlett Test:")
      message("Chi-square: ", round(bart$chisq, 3))
      message("df: ", bart$df)
      message("p-value: ", format.pval(bart$p.value))
    } else {
      message("Bartlett test not valid")
    }
  }

  kmo <- NULL
  if (!singular) {
    kmo <- tryCatch(psych::KMO(cm), error = function(e) NULL)
  }

  kmo_val <- NULL
  suitability <- NULL

  if (!is.null(kmo)) {
    kmo_val <- round(kmo$MSA, 3)
    suitability <- ifelse(
      kmo_val >= 0.6,
      "Suitable for PCA",
      "Not Suitable for PCA"
    )
  }

  if (verbose) {
    if (!is.null(kmo)) {
      message("KMO: ", kmo_val, " - ", suitability)
    } else {
      message("KMO not valid")
    }
  }

  p <- stats::prcomp(df, scale. = scale., center = center)

  load <- as.data.frame(p$rotation)
  scores <- as.data.frame(p$x)
  ve <- p$sdev^2 / sum(p$sdev^2)

    theme_pretty <- ggplot2::theme_minimal(base_family = "serif", base_size = 16) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", hjust = 0.5),
      axis.title = ggplot2::element_text(face = "bold"),
      axis.text = ggplot2::element_text(face = "bold", color = "black"),
      axis.line = ggplot2::element_line(color = "black"),
      axis.ticks = ggplot2::element_line(color = "black"),
      panel.grid.minor = ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(paste0("PC", seq_along(ve)), levels = paste0("PC", seq_along(ve))),
    Var = ve,
    CumVar = cumsum(ve)
  )

  scree <- ggplot2::ggplot(scree_data, ggplot2::aes(x = PC)) +
    ggplot2::geom_col(ggplot2::aes(y = Var), fill = "#4C72B0", width = 0.6, alpha = 0.85) +
    ggplot2::geom_line(ggplot2::aes(y = CumVar, group = 1), color = "#DD8452", linewidth = 1.2) +
    ggplot2::geom_point(ggplot2::aes(y = CumVar), color = "#DD8452", size = 3) +
    ggplot2::geom_text(
      ggplot2::aes(y = Var, label = paste0(round(Var * 100, 1), "%")),
      vjust = -0.5, size = 4, family = "serif", fontface = "bold"
    ) +
    ggplot2::scale_y_continuous(labels = scales::percent_format(), limits = c(0, 1)) +
    ggplot2::labs(
      title = "PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  if (ncol(scores) >= 2) {

    load2 <- load[, 1:2, drop = FALSE]
    load2$v <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]
    sf <- max(abs(score2))

    biplot <- ggplot2::ggplot() +
      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(PC1, PC2),
        size = 3, alpha = 0.75, color = "#2C7BB6"
      ) +
      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(x = 0, y = 0, xend = PC1 * sf, yend = PC2 * sf),
        arrow = grid::arrow(length = grid::unit(0.3, "cm")),
        color = "#D7191C", linewidth = 1.2
      ) +
      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(PC1 * sf, PC2 * sf, label = v),
        size = 5, color = "#D7191C",
        family = "serif", fontface = "bold"
      ) +
      ggplot2::labs(
        title = "PCA Biplot",
        x = "Component 1",
        y = "Component 2"
      ) +
      theme_pretty +
      ggplot2::coord_equal()

  } else {
    warning("Biplot not created: less than 2 principal components")
  }

  invisible(list(
    bartlett = bart,
    kmo = kmo,
    loadings = load,
    scores = scores,
    variance = ve,
    scree_plot = scree,
    biplot = if (exists("biplot")) biplot else NULL
  ))
}

utils::globalVariables(c("PC", "Var", "CumVar", "PC1", "PC2", "v"))

#' @keywords internal
#' @noRd
choose_K <- function(data,
                     threshold = 0.9,
                     verbose = FALSE){

  df <- as.data.frame(data)
  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  p <- stats::prcomp(df, scale. = TRUE)

  ve <- p$sdev^2 / sum(p$sdev^2)
  cumvar <- cumsum(ve)

  K_var <- which(cumvar >= threshold)[1]

  d1 <- diff(ve)
  d2 <- diff(d1)

  K_elbow <- if(length(d2) > 0) {
    which.min(d2) + 1
  } else {
    1
  }

  K <- min(K_var, K_elbow, ncol(df))

  if(verbose){
    message("\n--- Auto Selection of K ---")
    message("K (Variance >= ", threshold * 100, "%): ", K_var)
    message("K (Elbow): ", K_elbow)
    message("Selected K: ", K)
  }

  return(K)
}

#' @title Sparse Principal Component Regression (Sparse PCR)
#'
#' @description
#' Performs Sparse PCA followed by regression with automatic selection
#' of number of components. Includes PCA suitability tests, loadings,
#' regression models, scree plot, and biplot.
#'
#' @param data A data frame or matrix
#' @param y Response variable name (character)
#' @param K Number of components (optional)
#' @param threshold Variance threshold for automatic K selection
#' @param verbose Logical; print messages and summaries
#'
#' @return A list containing model outputs and PCA results
#'
#' @importFrom stats cor prcomp lm step complete.cases sd
#' @importFrom psych cortest.bartlett KMO
#' @importFrom elasticnet spca
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point geom_text
#' @importFrom ggplot2 scale_y_continuous labs theme_minimal theme
#' @importFrom ggplot2 element_text element_line element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#' @export
#'
#' @examples
#' data(mtcars)
#' Result <- sparse_pcreg(mtcars, y = "mpg")

sparse_pcreg <- function(data,
                         y,
                         K = NULL,
                         threshold = 0.9,
                         verbose = FALSE){

  if(!is.data.frame(data) && !is.matrix(data)){
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)

  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  if(!(y %in% colnames(df))){
    stop("Response variable not found")
  }

  Y <- df[[y]]

  if(!is.numeric(Y)){
    stop("Response must be numeric")
  }

  X <- df[, setdiff(names(df), y), drop = FALSE]

  if(ncol(X) < 2){
    stop("Need at least 2 predictors")
  }

  idx <- stats::complete.cases(X, Y)

  X <- X[idx, , drop = FALSE]
  Y <- Y[idx]

  if(nrow(X) < 2){
    stop("Not enough complete observations")
  }

  const <- vapply(X, function(x) stats::sd(x) == 0, logical(1))

  if(any(const)){

    if(verbose){
      message(
        "Removed constant predictors: ",
        paste(names(X)[const], collapse = ", ")
      )
    }

    X <- X[, !const, drop = FALSE]
  }

  if(ncol(X) < 2){
    stop("Need at least 2 non-constant predictors")
  }

  if(is.null(K)){
    K <- choose_K(X,
                  threshold = threshold,
                  verbose = verbose)
  }

  if(K > ncol(X)){
    stop("K cannot exceed number of predictors")
  }

  if(verbose){
    message("\nUsing K = ", K)
  }

  cm <- stats::cor(X)

  if(verbose){
    message("\n--- PCA Suitability ---")
  }

  singular <- tryCatch(
    base::det(cm) < 1e-10,
    error = function(e) TRUE
  )

  bart <- if(!singular){
    tryCatch(
      psych::cortest.bartlett(cm, n = nrow(X)),
      error = function(e) NULL
    )
  } else {
    NULL
  }

  if(!is.null(bart)){

    if(verbose){
      message("\nBartlett Test:")
      message("Chi-square: ", round(bart$chisq, 3))
      message("df: ", bart$df)
      message("p-value: ", format.pval(bart$p.value))
    }

  } else {

    if(verbose){
      message("\nBartlett test not valid")
    }
  }

  kmo <- if(!singular){
    tryCatch(
      psych::KMO(cm),
      error = function(e) NULL
    )
  } else {
    NULL
  }

  if(!is.null(kmo)){

    kmo_val <- round(kmo$MSA, 3)

    suitability <- ifelse(
      kmo_val >= 0.6,
      "Suitable for PCA",
      "Not Suitable for PCA"
    )

    if(verbose){
      message("\nKMO: ", kmo_val, " - ", suitability)
    }

  } else {

    if(verbose){
      message("\nKMO not valid")
    }
  }

  X_scaled <- scale(X)

  sp <- elasticnet::spca(
    X_scaled,
    K = K,
    type = "predictor",
    sparse = "varnum",
    para = rep(ceiling(ncol(X) / 2), K)
  )

  load <- as.matrix(sp$loadings)

  colnames(load) <- paste0("PC", seq_len(K))
  rownames(load) <- colnames(X)

  load_df <- as.data.frame(load)

  scores <- as.data.frame(X_scaled %*% load)

  p_full <- stats::prcomp(X, scale. = TRUE)

  ve_full <- p_full$sdev^2 / sum(p_full$sdev^2)

  if(verbose){
    message("\n--- PCA Summary ---")
    print(summary(p_full))

    message("\n--- Sparse PCA Loadings ---")
    print(round(load_df, 3))
  }

  reg_data <- data.frame(Y = Y, scores)

  full_model <- stats::lm(Y ~ ., data = reg_data)

  if(verbose){
    message("\n--- FULL SPARSE PCR MODEL ---")
    print(summary(full_model))
  }

  step_model <- tryCatch(
    stats::step(full_model, direction = "backward", trace = 0),
    error = function(e) full_model
  )

  if(verbose){
    message("\n--- BACKWARD SELECTED MODEL ---")
    print(summary(step_model))
  }

  theme_pretty <-
    ggplot2::theme_minimal(
      base_family = "serif",
      base_size = 16
    ) +
    ggplot2::theme(
      plot.title =
        ggplot2::element_text(
          face = "bold",
          hjust = 0.5
        ),
      axis.title =
        ggplot2::element_text(face = "bold"),
      axis.text =
        ggplot2::element_text(
          face = "bold",
          color = "black"
        ),
      axis.line.x =
        ggplot2::element_line(color = "black"),
      axis.line.y =
        ggplot2::element_line(color = "black"),
      axis.ticks =
        ggplot2::element_line(color = "black"),
      panel.grid.minor =
        ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(
      paste0("PC", seq_along(ve_full)),
      levels = paste0("PC", seq_along(ve_full))
    ),
    Var = ve_full,
    CumVar = cumsum(ve_full)
  )

  scree <- ggplot2::ggplot(
    scree_data,
    ggplot2::aes(x = PC)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(y = Var),
      fill = "#4C72B0",
      width = 0.6,
      alpha = 0.85
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = CumVar, group = 1),
      color = "#DD8452",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = CumVar),
      color = "#DD8452",
      size = 3
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        y = Var,
        label = paste0(round(Var * 100, 1), "%")
      ),
      vjust = -0.5,
      size = 4,
      family = "serif",
      fontface = "bold"
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::percent_format(),
      limits = c(0, 1)
    ) +
    ggplot2::labs(
      title = "Sparse PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  if(ncol(scores) >= 2){

    load2 <- load_df[, 1:2, drop = FALSE]
    load2$v <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]

    sf <- max(abs(score2))

    biplot <- ggplot2::ggplot() +

      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(PC1, PC2),
        size = 3,
        alpha = 0.75,
        color = "#2C7BB6"
      ) +

      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(
          x = 0,
          y = 0,
          xend = PC1 * sf,
          yend = PC2 * sf
        ),
        arrow = grid::arrow(
          length = grid::unit(0.3, "cm")
        ),
        color = "#D7191C",
        linewidth = 1.2
      ) +

      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(
          PC1 * sf,
          PC2 * sf,
          label = v
        ),
        size = 5,
        color = "#D7191C",
        family = "serif",
        fontface = "bold"
      ) +

      ggplot2::labs(
        title = "Sparse PCA Biplot",
        x = "Component 1",
        y = "Component 2"
      ) +

      theme_pretty +
      ggplot2::coord_equal()

    gridExtra::grid.arrange(
      scree,
      biplot,
      ncol = 2
    )

  } else {

    warning("Biplot not created: less than 2 components")

    print(scree)
  }

  invisible(list(
    K = K,
    bartlett = bart,
    kmo = kmo,
    loadings = load_df,
    scores = scores,
    variance = ve_full,
    full_model = summary(full_model),
    backward_model = summary(step_model),
    scree_plot = scree,
    biplot = if (exists("biplot")) biplot else NULL
  ))
}

utils::globalVariables(c("PC", "Var", "CumVar", "PC1", "PC2", "v"))

#' @keywords internal
#' @noRd
choose_K <- function(data,
                     threshold = 0.9,
                     verbose = FALSE){

  df <- as.data.frame(data)
  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  p <- stats::prcomp(df, scale. = TRUE)

  ve <- p$sdev^2 / sum(p$sdev^2)
  cumvar <- cumsum(ve)

  K_var <- which(cumvar >= threshold)[1]

  d1 <- diff(ve)
  d2 <- diff(d1)

  K_elbow <- if(length(d2) > 0){
    which.min(d2) + 1
  } else {
    1
  }

  K <- min(K_var, K_elbow, ncol(df))

  if(verbose){
    message("\n--- Auto Selection of K ---")
    message("K (Variance >= ", threshold * 100, "%): ", K_var)
    message("K (Elbow): ", K_elbow)
    message("Selected K: ", K)
  }

  return(K)
}

#' @title Robust Principal Component Regression (Robust PCR)
#'
#' @description
#' Performs Robust PCA using Minimum Covariance Determinant (MCD)
#' followed by regression on robust principal components.
#' Includes PCA suitability tests, summary, loadings,
#' scree plot, biplot, and regression models.
#'
#' @param data A data frame or matrix
#' @param y Response variable name (character)
#' @param K Number of components (optional)
#' @param threshold Variance threshold for automatic K selection
#' @param verbose Logical; print detailed output
#'
#' @return A list containing Robust PCR outputs
#'
#' @importFrom stats prcomp lm step complete.cases sd cor
#' @importFrom robustbase covMcd
#' @importFrom psych cortest.bartlett KMO
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point geom_text
#' @importFrom ggplot2 scale_y_continuous labs theme_minimal theme
#' @importFrom ggplot2 element_text element_line element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#' @export
#'
#' @examples
#' data(mtcars)
#' Result <- robust_pcreg(mtcars, y = "mpg")

robust_pcreg <- function(data,
                         y,
                         K = NULL,
                         threshold = 0.9,
                         verbose = FALSE){

  if(!is.data.frame(data) && !is.matrix(data)){
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)
  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  if(!(y %in% colnames(df))){
    stop("Response variable not found")
  }

  Y <- df[[y]]

  if(!is.numeric(Y)){
    stop("Response must be numeric")
  }

  X <- df[, setdiff(names(df), y), drop = FALSE]

  if(ncol(X) < 2){
    stop("Need at least 2 predictors")
  }

  idx <- stats::complete.cases(X, Y)
  X <- X[idx, , drop = FALSE]
  Y <- Y[idx]

  if(nrow(X) < 2){
    stop("Not enough complete observations")
  }

  const <- vapply(X, function(x) stats::sd(x) == 0, logical(1))

  if(any(const)){
    if(verbose){
      message("Removed constant predictors: ",
              paste(names(X)[const], collapse = ", "))
    }
    X <- X[, !const, drop = FALSE]
  }

  if(ncol(X) < 2){
    stop("Need at least 2 non-constant predictors")
  }

  if(is.null(K)){
    K <- choose_K(X, threshold = threshold, verbose = verbose)
  }

  if(K > ncol(X)){
    stop("K cannot exceed number of predictors")
  }

  if(verbose){
    message("\nUsing K = ", K)
  }

  cm <- stats::cor(X)

  if(verbose){
    message("\n--- PCA Suitability ---")
  }

  singular <- tryCatch(base::det(cm) < 1e-10,
                       error = function(e) TRUE)

  bart <- if(!singular){
    tryCatch(psych::cortest.bartlett(cm, n = nrow(X)),
             error = function(e) NULL)
  } else NULL

  if(!is.null(bart)){
    if(verbose){
      message("\nBartlett Test:")
      message("Chi-square: ", round(bart$chisq, 3))
      message("df: ", bart$df)
      message("p-value: ", format.pval(bart$p.value))
    }
  } else if(verbose){
    message("\nBartlett test not valid")
  }

  kmo <- if(!singular){
    tryCatch(psych::KMO(cm),
             error = function(e) NULL)
  } else NULL

  if(!is.null(kmo)){
    kmo_val <- round(kmo$MSA, 3)
    if(verbose){
      message("\nKMO: ", kmo_val,
              " - ",
              ifelse(kmo_val >= 0.6,
                     "Suitable for PCA",
                     "Not Suitable for PCA"))
    }
  } else if(verbose){
    message("\nKMO not valid")
  }

  mcd <- robustbase::covMcd(X)

  eig <- eigen(mcd$cov, symmetric = TRUE)
  eig_vals <- eig$values
  eig_vecs <- eig$vectors

  ve_full <- eig_vals / sum(eig_vals)

  pca_summary <- rbind(
    "Standard deviation" = sqrt(eig_vals),
    "Proportion of Variance" = ve_full,
    "Cumulative Proportion" = cumsum(ve_full)
  )

  colnames(pca_summary) <- paste0("PC", seq_len(ncol(pca_summary)))

  if(verbose){
    message("\n--- PCA Summary ---")
    print(round(pca_summary, 4))
  }

  load <- eig_vecs[, 1:K, drop = FALSE]
  colnames(load) <- paste0("PC", seq_len(K))
  rownames(load) <- colnames(X)
  load_df <- as.data.frame(load)

  scores <- scale(X, center = mcd$center, scale = FALSE) %*% load
  scores <- as.data.frame(scores)

  reg_data <- data.frame(Y = Y, scores)

  full_model <- stats::lm(Y ~ ., data = reg_data)

  step_model <- tryCatch(
    stats::step(full_model, direction = "backward", trace = 0),
    error = function(e) full_model
  )

  if(verbose){
    message("\n--- FULL ROBUST PCR MODEL ---")
    print(summary(full_model))

    message("\n--- BACKWARD SELECTED MODEL ---")
    print(summary(step_model))
  }

  theme_pretty <-
    ggplot2::theme_minimal(base_family = "serif", base_size = 16) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", hjust = 0.5),
      axis.title = ggplot2::element_text(face = "bold"),
      axis.text = ggplot2::element_text(face = "bold", color = "black"),
      axis.line.x = ggplot2::element_line(color = "black"),
      axis.line.y = ggplot2::element_line(color = "black"),
      axis.ticks = ggplot2::element_line(color = "black"),
      panel.grid.minor = ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(paste0("PC", seq_along(ve_full))),
    Var = ve_full,
    CumVar = cumsum(ve_full)
  )

  scree <- ggplot2::ggplot(scree_data, ggplot2::aes(x = PC)) +
    ggplot2::geom_col(ggplot2::aes(y = Var), width = 0.7) +
    ggplot2::geom_line(ggplot2::aes(y = CumVar, group = 1),
                       color = "#D55E00", linewidth = 1.2) +
    ggplot2::geom_point(ggplot2::aes(y = CumVar),
                        color = "#D55E00", size = 3) +
    ggplot2::geom_text(ggplot2::aes(y = Var,
                                    label = paste0(round(Var * 100, 1), "%")),
                       vjust = -0.5, size = 4, fontface = "bold") +
    ggplot2::scale_y_continuous(labels = scales::percent_format(),
                                limits = c(0, 1)) +
    ggplot2::labs(title = "Robust PCA Scree Plot",
                  x = "Principal Components",
                  y = "Variance Explained") +
    theme_pretty

  if(ncol(scores) >= 2){

    load2 <- load_df[, 1:2, drop = FALSE]
    load2$v <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]
    sf <- max(abs(score2))

    biplot <- ggplot2::ggplot() +
      ggplot2::geom_point(data = score2,
                          ggplot2::aes(PC1, PC2),
                          size = 3, alpha = 0.8) +
      ggplot2::geom_segment(data = load2,
                            ggplot2::aes(x = 0, y = 0,
                                         xend = PC1 * sf,
                                         yend = PC2 * sf),
                            arrow = grid::arrow(length = grid::unit(0.3, "cm")),
                            linewidth = 1.2) +
      ggrepel::geom_text_repel(data = load2,
                               ggplot2::aes(PC1 * sf, PC2 * sf, label = v),
                               size = 5, fontface = "bold") +
      ggplot2::labs(title = "Robust PCA Biplot",
                    x = "Component 1",
                    y = "Component 2") +
      theme_pretty +
      ggplot2::coord_equal()

    gridExtra::grid.arrange(scree, biplot, ncol = 2)

  } else {

    warning("Biplot requires at least 2 PCs")
    print(scree)
  }

  invisible(list(
    K = K,
    bartlett = bart,
    kmo = kmo,
    loadings = load_df,
    scores = scores,
    variance = ve_full,
    full_model = summary(full_model),
    backward_model = summary(step_model),
    scree_plot = scree,
    biplot = if (exists("biplot")) biplot else NULL
  ))
}

utils::globalVariables(c("v", "PC1", "PC2"))

#' @title Supervised Principal Component Analysis (SPCA)
#' @description True SPCA (Bair et al.): screening + PCA
#'
#' @param data Numeric data frame or matrix
#' @param response Numeric response vector
#' @param K Number of components (auto if NULL)
#' @param threshold Variance threshold for K selection
#' @param verbose Logical; if TRUE prints progress messages
#'
#' @return List of SPCA results
#'
#' @importFrom stats prcomp cor complete.cases sd
#' @importFrom psych cortest.bartlett KMO
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point
#' @importFrom ggplot2 geom_text scale_y_continuous labs
#' @importFrom ggplot2 theme_minimal theme
#' @importFrom ggplot2 element_text element_line
#' @importFrom ggplot2 element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#' @export
#'
#' @examples
#' data(mtcars)
#' Result <- supervised_pca(
#'   mtcars[, -1],
#'   mtcars$mpg
#' )

supervised_pca <- function(data,
                           response,
                           K = NULL,
                           threshold = 0.9,
                           verbose = FALSE) {

  if (!is.data.frame(data) &&
      !is.matrix(data)) {
    stop("data must be a data frame or matrix")
  }

  if (!is.numeric(response)) {
    stop("response must be numeric")
  }

  df <- as.data.frame(data)

  df <- df[, vapply(
    df,
    is.numeric,
    logical(1)
  ), drop = FALSE]

  if (ncol(df) < 2) {
    stop("Need at least 2 numeric predictors")
  }

  if (length(response) != nrow(df)) {
    stop("response length must match rows")
  }

  idx <- stats::complete.cases(df, response)

  df <- df[idx, , drop = FALSE]
  y <- response[idx]

  if (nrow(df) < 2) {
    stop("Not enough complete observations")
  }

  cors <- vapply(
    df,
    function(x) {

      if (stats::sd(x) == 0) {
        return(0)
      }

      stats::cor(x, y)
    },
    numeric(1)
  )

  keep <- abs(cors) > 0.2

  if (sum(keep) < 2) {
    stop("Too few variables selected")
  }

  df_sel <- df[, keep, drop = FALSE]

  if (verbose) {

    message("\n--- Supervised Selection ---")

    message(
      "Selected Variables: ",
      paste(names(df_sel),
            collapse = ", ")
    )
  }

  if (is.null(K)) {

    p_tmp <- stats::prcomp(
      df_sel,
      scale. = TRUE
    )

    ve <- p_tmp$sdev^2 /
      sum(p_tmp$sdev^2)

    K <- which(
      cumsum(ve) >= threshold
    )[1]

    if (verbose) {

      message(
        "\nSelected K: ",
        K
      )
    }
  }

  if (!is.numeric(K) ||
      length(K) != 1) {
    stop("K must be a single numeric value")
  }

  K <- as.integer(K)

  if (K < 1) {
    stop("K must be >= 1")
  }

  if (K > ncol(df_sel)) {
    stop("K cannot exceed variables")
  }

  cm <- stats::cor(df_sel)

  singular <- tryCatch(
    base::det(cm) < 1e-10,
    error = function(e) TRUE
  )

  bartlett <- NULL
  kmo <- NULL

  if (!singular) {

    bartlett <- tryCatch(
      psych::cortest.bartlett(
        cm,
        n = nrow(df_sel)
      ),
      error = function(e) NULL
    )

    kmo <- tryCatch(
      psych::KMO(cm),
      error = function(e) NULL
    )
  }

  if (verbose) {

    message("\n--- PCA Suitability ---")

    if (!is.null(bartlett)) {

      message(
        "Bartlett p-value: ",
        format.pval(
          bartlett$p.value
        )
      )

    } else {

      message(
        "Bartlett test not valid"
      )
    }

    if (!is.null(kmo)) {

      message(
        "KMO: ",
        round(kmo$MSA, 3)
      )

    } else {

      message("KMO not valid")
    }
  }

  p <- stats::prcomp(
    df_sel,
    scale. = TRUE
  )

  ve_full <- p$sdev^2 /
    sum(p$sdev^2)

  pca_summary <- summary(p)

  load <- p$rotation

  scores <- as.data.frame(
    p$x[, 1:K, drop = FALSE]
  )

  theme_pretty <-
    ggplot2::theme_minimal(
      base_family = "serif",
      base_size = 16
    ) +
    ggplot2::theme(
      plot.title =
        ggplot2::element_text(
          face = "bold",
          hjust = 0.5
        ),
      axis.title =
        ggplot2::element_text(
          face = "bold"
        ),
      axis.text =
        ggplot2::element_text(
          face = "bold",
          color = "black"
        ),
      axis.line =
        ggplot2::element_line(
          color = "black"
        ),
      axis.ticks =
        ggplot2::element_line(
          color = "black"
        ),
      panel.grid.minor =
        ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(
      paste0(
        "PC",
        seq_along(ve_full)
      ),
      levels = paste0(
        "PC",
        seq_along(ve_full)
      )
    ),
    Var = ve_full,
    CumVar = cumsum(ve_full)
  )

  scree <- ggplot2::ggplot(
    scree_data,
    ggplot2::aes(x = PC)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(y = Var),
      fill = "#4C72B0",
      width = 0.6
    ) +
    ggplot2::geom_line(
      ggplot2::aes(
        y = CumVar,
        group = 1
      ),
      color = "#D55E00",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = CumVar),
      color = "#D55E00",
      size = 3
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        y = Var,
        label = paste0(
          round(Var * 100, 1),
          "%"
        )
      ),
      vjust = -0.5,
      family = "serif",
      fontface = "bold",
      size = 4
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::percent_format(),
      limits = c(0, 1)
    ) +
    ggplot2::labs(
      title = "Supervised PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  biplot <- NULL

  if (ncol(scores) >= 2) {

    load2 <- as.data.frame(
      load[, 1:2, drop = FALSE]
    )

    load2$v <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]

    sf <- max(abs(score2))

    biplot <- ggplot2::ggplot() +

      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(PC1, PC2),
        size = 3,
        alpha = 0.75,
        color = "#2C7BB6"
      ) +

      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(
          x = 0,
          y = 0,
          xend = PC1 * sf,
          yend = PC2 * sf
        ),
        arrow = grid::arrow(
          length = grid::unit(
            0.3,
            "cm"
          )
        ),
        color = "#D7191C",
        linewidth = 1.2
      ) +

      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(
          PC1 * sf,
          PC2 * sf,
          label = v
        ),
        size = 5,
        color = "#D7191C",
        family = "serif",
        fontface = "bold"
      ) +

      ggplot2::labs(
        title = "Supervised PCA Biplot",
        x = "Component 1",
        y = "Component 2"
      ) +

      theme_pretty +

      ggplot2::coord_equal()

  } else {

    warning("Biplot not created")
  }

  invisible(list(
    K = K,
    selected_vars = names(df_sel),
    bartlett = bartlett,
    kmo = kmo,
    summary = pca_summary,
    loadings = load,
    scores = scores,
    variance = ve_full,
    scree_plot = scree,
    biplot = biplot
  ))
}

utils::globalVariables(c(
  "PC", "Var", "CumVar",
  "PC1", "PC2", "v"
))

#' @title Supervised Principal Component Regression
#'
#' @description
#' Performs supervised feature selection followed by
#' Principal Component Regression (PCR).
#' Includes Bartlett's test, KMO measure,
#' scree plot, biplot, and regression models.
#'
#' @param data A data frame or matrix
#' @param response Response variable name
#' @param K Number of principal components
#' @param threshold Cumulative variance threshold
#' for automatic component selection
#' @param verbose Logical; print summaries?
#'
#' @return A list containing PCA results,
#' regression models, diagnostics, and plots.
#'
#' @importFrom stats cor prcomp lm step complete.cases sd
#' @importFrom psych cortest.bartlett KMO
#' @importFrom ggplot2 ggplot aes geom_col geom_line
#' @importFrom ggplot2 geom_point geom_text
#' @importFrom ggplot2 scale_y_continuous labs
#' @importFrom ggplot2 theme_minimal theme
#' @importFrom ggplot2 element_text element_line
#' @importFrom ggplot2 element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#'
#' @export
#'
#' @examples
#' data(mtcars)
#' result <- supervised_pcreg(
#'   mtcars,
#'   response = "mpg"
#' )

supervised_pcreg <- function(data,
                             response,
                             K = NULL,
                             threshold = 0.90,
                             verbose = FALSE) {

  if (!is.data.frame(data) &&
      !is.matrix(data)) {
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)

  df <- df[, vapply(
    df,
    is.numeric,
    logical(1)
  ), drop = FALSE]

  if (!is.character(response) ||
      length(response) != 1) {
    stop(
      "response must be a single character string"
    )
  }

  if (!(response %in% names(df))) {
    stop("Response variable not found")
  }

  Y <- df[[response]]

  X <- df[, setdiff(
    names(df),
    response
  ), drop = FALSE]

  if (ncol(X) < 2) {
    stop("Need at least two predictors")
  }

  idx <- stats::complete.cases(X, Y)

  X <- X[idx, , drop = FALSE]
  Y <- Y[idx]

  if (nrow(X) < 2) {
    stop("Not enough complete observations")
  }

  const <- vapply(
    X,
    function(x) stats::sd(x) == 0,
    logical(1)
  )

  if (any(const)) {

    if (verbose) {
      message(
        "Removed constant predictors: ",
        paste(names(X)[const],
              collapse = ", ")
      )
    }

    X <- X[, !const, drop = FALSE]
  }

  if (ncol(X) < 2) {
    stop(
      "Need at least two non-constant predictors"
    )
  }

  cors <- vapply(
    X,
    function(x) {

      r <- stats::cor(
        x,
        Y,
        use = "complete.obs"
      )

      ifelse(is.na(r), 0, r)
    },
    numeric(1)
  )

  keep <- abs(cors) > 0.20

  if (sum(keep) < 2) {
    stop("Too few variables selected")
  }

  X_sel <- X[, keep, drop = FALSE]

  if (verbose) {

    message("\n--- Supervised Selection ---")

    message(
      paste(
        names(X_sel),
        collapse = ", "
      )
    )
  }

  cm <- stats::cor(X_sel)

  singular <- tryCatch(
    base::det(cm) < 1e-10,
    error = function(e) TRUE
  )

  bart <- NULL
  kmo <- NULL

  if (!singular) {

    bart <- tryCatch(
      psych::cortest.bartlett(
        cm,
        n = nrow(X_sel)
      ),
      error = function(e) NULL
    )

    kmo <- tryCatch(
      psych::KMO(cm),
      error = function(e) NULL
    )
  }

  if (verbose) {

    message("\n--- PCA Suitability ---")

    if (!is.null(bart)) {

      message(
        "Bartlett p-value: ",
        format.pval(bart$p.value)
      )

    } else {

      message("Bartlett test not valid")
    }

    if (!is.null(kmo)) {

      message(
        "KMO: ",
        round(kmo$MSA, 3)
      )

    } else {

      message("KMO not valid")
    }
  }

  choose_K <- function(data,
                       threshold = 0.90) {

    pca0 <- stats::prcomp(
      data,
      scale. = TRUE
    )

    ve0 <- pca0$sdev^2 /
      sum(pca0$sdev^2)

    which(cumsum(ve0) >= threshold)[1]
  }

  if (is.null(K)) {

    K <- choose_K(
      X_sel,
      threshold = threshold
    )
  }

  if (!is.numeric(K) ||
      length(K) != 1) {
    stop("K must be a single numeric value")
  }

  K <- as.integer(K)

  if (K < 1) {
    stop("K must be >= 1")
  }

  if (K > ncol(X_sel)) {
    stop(
      "K cannot exceed number of selected predictors"
    )
  }

  if (verbose) {
    message("\nUsing K = ", K)
  }

  pca <- stats::prcomp(
    X_sel,
    scale. = TRUE
  )

  variance <- pca$sdev^2 /
    sum(pca$sdev^2)

  pca_summary <- summary(pca)

  loadings <- as.data.frame(
    pca$rotation[, 1:K, drop = FALSE]
  )

  scores <- as.data.frame(
    pca$x[, 1:K, drop = FALSE]
  )

  if (verbose) {

    message("\n--- PCA Summary ---")

    print(pca_summary$importance)

    message("\n--- PCA Loadings ---")

    print(round(loadings, 3))
  }

  reg_data <- data.frame(
    Y = Y,
    scores
  )

  full_model <- stats::lm(
    Y ~ .,
    data = reg_data
  )

  backward_model <- tryCatch(
    stats::step(
      full_model,
      direction = "backward",
      trace = 0
    ),
    error = function(e) full_model
  )

  if (verbose) {

    message("\n--- Full Model ---")

    print(summary(full_model))

    message(
      "\n--- Backward Selected Model ---"
    )

    print(summary(backward_model))
  }

  theme_pretty <-
    ggplot2::theme_minimal(
      base_family = "serif",
      base_size = 14
    ) +
    ggplot2::theme(
      plot.title =
        ggplot2::element_text(
          face = "bold",
          hjust = 0.5
        ),
      axis.title =
        ggplot2::element_text(
          face = "bold"
        ),
      axis.text =
        ggplot2::element_text(
          face = "bold",
          color = "black"
        ),
      axis.line =
        ggplot2::element_line(
          color = "black"
        ),
      axis.ticks =
        ggplot2::element_line(
          color = "black"
        ),
      panel.grid.minor =
        ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(
      paste0(
        "PC",
        seq_along(variance)
      ),
      levels = paste0(
        "PC",
        seq_along(variance)
      )
    ),
    Var = variance,
    CumVar = cumsum(variance)
  )

  scree_plot <- ggplot2::ggplot(
    scree_data,
    ggplot2::aes(x = PC)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(y = Var),
      fill = "#4C72B0",
      width = 0.6
    ) +
    ggplot2::geom_line(
      ggplot2::aes(
        y = CumVar,
        group = 1
      ),
      color = "#D55E00",
      linewidth = 1
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = CumVar),
      color = "#D55E00",
      size = 2.5
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        y = Var,
        label = paste0(
          round(Var * 100, 1),
          "%"
        )
      ),
      vjust = -0.4,
      size = 4,
      family = "serif",
      fontface = "bold"
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::percent_format(),
      limits = c(0, 1)
    ) +
    ggplot2::labs(
      title = "Supervised PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  biplot_obj <- NULL

  if (K >= 2) {

    load2 <- loadings[, 1:2, drop = FALSE]

    load2$v <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]

    scale_factor <- max(abs(score2))

    biplot_obj <- ggplot2::ggplot() +

      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(PC1, PC2),
        color = "#2C7BB6",
        size = 3,
        alpha = 0.7
      ) +

      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(
          x = 0,
          y = 0,
          xend = PC1 * scale_factor,
          yend = PC2 * scale_factor
        ),
        arrow = grid::arrow(
          length = grid::unit(
            0.25,
            "cm"
          )
        ),
        color = "#D7191C",
        linewidth = 1
      ) +

      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(
          PC1 * scale_factor,
          PC2 * scale_factor,
          label = v
        ),
        color = "#D7191C",
        family = "serif",
        fontface = "bold",
        size = 4
      ) +

      ggplot2::labs(
        title = "Supervised PCA Biplot",
        x = "PC1",
        y = "PC2"
      ) +

      ggplot2::coord_equal() +
      theme_pretty

    gridExtra::grid.arrange(
      scree_plot,
      biplot_obj,
      ncol = 2
    )

  } else {

    warning(
      "Biplot requires at least two components"
    )

    print(scree_plot)
  }

  invisible(list(
    K = K,
    selected_variables = names(X_sel),
    pca = pca,
    bartlett = bart,
    kmo = kmo,
    loadings = loadings,
    scores = scores,
    variance = variance,
    full_model = summary(full_model),
    backward_model =
      summary(backward_model),
    scree_plot = scree_plot,
    biplot = biplot_obj
  ))
}

utils::globalVariables(c("varname"))

#' @keywords internal
#' @noRd
choose_K <- function(data,
                     threshold = 0.9,
                     verbose = FALSE) {

  df <- as.data.frame(data)

  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  p <- stats::prcomp(df, scale. = TRUE)

  ve <- p$sdev^2 / sum(p$sdev^2)

  cumvar <- cumsum(ve)

  K_var <- which(cumvar >= threshold)[1]

  d1 <- diff(ve)
  d2 <- diff(d1)

  K_elbow <- if (length(d2) > 0) {
    which.min(d2) + 1
  } else {
    1
  }

  K <- min(K_var, K_elbow)

  if (verbose) {
    message("--- Auto Selection of K ---")
    message("K (Variance >= ", threshold * 100, "%): ", K_var)
    message("K (Elbow): ", K_elbow)
    message("Selected K: ", K)
  }

  return(K)
}

#' @title Sparse Principal Component Analysis (Sparse PCA)
#'
#' @description
#' Performs Sparse PCA using elastic net regularisation with custom scree plot and biplot.
#'
#' @param data Numeric data frame or matrix
#' @param K Number of components (auto if NULL)
#' @param threshold Variance threshold for automatic K selection
#' @param verbose Logical; prints diagnostic messages
#'
#' @return List with PCA results
#'
#' @importFrom stats prcomp cor complete.cases
#' @importFrom psych cortest.bartlett KMO
#' @importFrom elasticnet spca
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point geom_text
#' @importFrom ggplot2 scale_y_continuous labs theme_minimal theme
#' @importFrom ggplot2 element_text element_line element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#' @export
#' @examples {
#' data(mtcars)
#' Result <- sparse_pca(mtcars)
#' }

sparse_pca <- function(data,
                       K = NULL,
                       threshold = 0.9,
                       verbose = FALSE) {

  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)

  df <- df[, vapply(df, is.numeric, logical(1)), drop = FALSE]

  if (ncol(df) < 2) {
    stop("Need at least 2 numeric variables")
  }

  df <- df[stats::complete.cases(df), , drop = FALSE]

  if (nrow(df) < 2) {
    stop("Not enough complete observations")
  }

  if (is.null(K)) {

    K <- choose_K(
      data = df,
      threshold = threshold
    )

  }
  if (K > ncol(df)) {
    stop("K cannot exceed number of variables")
  }

  if (verbose) {
    message("Using K = ", K)
  }

  # FIXED: define X and cm
  X <- scale(df)
  cm <- stats::cor(df)

  singular <- tryCatch(base::det(cm) < 1e-10,
                       error = function(e) TRUE)

  bart <- if (!singular) {
    tryCatch(
      psych::cortest.bartlett(cm, n = nrow(df)),
      error = function(e) NULL
    )
  } else NULL

  if (!is.null(bart)) {
    if (verbose) {
      message("Bartlett Test:")
      message("Chi-square: ", round(bart$chisq, 3))
      message("df: ", bart$df)
      message("p-value: ", format.pval(bart$p.value))
    }
  } else {
    if (verbose) {
      message("Bartlett test not valid")
    }
  }

  kmo <- if (!singular) {
    tryCatch(psych::KMO(cm),
             error = function(e) NULL)
  } else NULL

  if (!is.null(kmo)) {

    kmo_val <- round(kmo$MSA, 3)

    suitability <- ifelse(
      kmo_val >= 0.6,
      "Suitable for PCA",
      "Not Suitable for PCA"
    )

    if (verbose) {
      message("KMO: ", kmo_val, " - ", suitability)
    }

  } else {

    if (verbose) {
      message("KMO not valid")
    }

  }

  sp <- elasticnet::spca(
    X,
    K = K,
    type = "predictor",
    sparse = "varnum",
    para = rep(ceiling(ncol(X) / 2), K)
  )

  load <- as.matrix(sp$loadings)

  colnames(load) <- paste0("PC", seq_len(K))
  rownames(load) <- colnames(df)

  load <- as.data.frame(load)

  scores <- as.data.frame(X %*% as.matrix(load))

  p_full <- stats::prcomp(df, scale. = TRUE)

  ve_full <- p_full$sdev^2 / sum(p_full$sdev^2)

  theme_pretty <- ggplot2::theme_minimal(
    base_family = "serif",
    base_size = 16
  ) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(
        face = "bold",
        hjust = 0.5
      ),
      axis.title = ggplot2::element_text(face = "bold"),
      axis.text = ggplot2::element_text(
        face = "bold",
        color = "black"
      ),
      axis.line.x = ggplot2::element_line(color = "black"),
      axis.line.y = ggplot2::element_line(color = "black"),
      axis.ticks = ggplot2::element_line(color = "black"),
      panel.grid.minor = ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(
      paste0("PC", seq_along(ve_full)),
      levels = paste0("PC", seq_along(ve_full))
    ),
    Var = ve_full,
    CumVar = cumsum(ve_full)
  )

  scree <- ggplot2::ggplot(
    scree_data,
    ggplot2::aes(x = PC)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(y = Var),
      fill = "#4C72B0",
      width = 0.6,
      alpha = 0.85
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = CumVar, group = 1),
      color = "#DD8452",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = CumVar),
      color = "#DD8452",
      size = 3
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        y = Var,
        label = paste0(round(Var * 100, 1), "%")
      ),
      vjust = -0.5,
      size = 4,
      family = "serif",
      fontface = "bold"
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::percent_format(),
      limits = c(0, 1)
    ) +
    ggplot2::labs(
      title = "Sparse PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  if (ncol(scores) >= 2) {

    load2 <- load[, 1:2, drop = FALSE]
    load2$varname <- rownames(load2)

    score2 <- scores[, 1:2, drop = FALSE]

    sf <- max(abs(score2))

    biplot <- ggplot2::ggplot() +
      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(PC1, PC2),
        size = 3,
        alpha = 0.75,
        color = "#2C7BB6"
      ) +
      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(
          x = 0,
          y = 0,
          xend = PC1 * sf,
          yend = PC2 * sf
        ),
        arrow = grid::arrow(
          length = grid::unit(0.3, "cm")
        ),
        color = "#D7191C",
        linewidth = 1.2
      ) +
      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(
          PC1 * sf,
          PC2 * sf,
          label = varname
        )
      ) +
      ggplot2::labs(
        title = "Sparse PCA Biplot",
        x = "Component 1",
        y = "Component 2"
      ) +
      theme_pretty +
      ggplot2::coord_equal()

  } else {

    warning("Biplot not created: less than 2 components")

  }

  invisible(list(
    K = K,
    bartlett = bart,
    kmo = kmo,
    loadings = load,
    scores = scores,
    variance = ve_full,
    scree_plot = scree,
    biplot = if (exists("biplot")) biplot else NULL
  ))
}

utils::globalVariables(c(
  "PC", "Var", "CumVar",
  "PC1", "PC2", "v"
))

#' @title Principal Component Regression (PCR)
#'
#' @description
#' Performs Principal Component Analysis (PCA) on predictors
#' followed by regression using selected principal components.
#' Includes Bartlett's test, KMO measure,
#' scree plot, biplot, and backward model selection.
#'
#' @param data A data frame or matrix
#' @param y Response variable name
#' @param scale. Logical; should variables be scaled?
#' @param center Logical; should variables be centered?
#' @param verbose Logical; print summaries?
#'
#' @return A list containing PCA results,
#' regression models, diagnostics, and plots.
#'
#' @importFrom stats cor prcomp lm step complete.cases
#' @importFrom psych cortest.bartlett KMO
#' @importFrom ggplot2 ggplot aes geom_col geom_line geom_point
#' @importFrom ggplot2 geom_text scale_y_continuous labs
#' @importFrom ggplot2 theme_minimal theme
#' @importFrom ggplot2 element_text element_line
#' @importFrom ggplot2 element_blank coord_equal
#' @importFrom ggrepel geom_text_repel
#' @importFrom gridExtra grid.arrange
#' @importFrom scales percent_format
#' @importFrom grid unit arrow
#'
#' @export
#'
#' @examples
#' data(mtcars)
#' result <- pcreg(mtcars, y = "mpg")

pcreg <- function(data,
                  y,
                  scale. = TRUE,
                  center = TRUE,
                  verbose = FALSE) {

  if (!is.data.frame(data) &&
      !is.matrix(data)) {
    stop("data must be a data frame or matrix")
  }

  df <- as.data.frame(data)

  df <- df[, vapply(
    df,
    is.numeric,
    logical(1)
  ), drop = FALSE]

  if (!is.character(y) ||
      length(y) != 1) {
    stop("y must be a single character string")
  }

  if (!(y %in% names(df))) {
    stop("Response variable not found")
  }

  X <- df[, setdiff(
    names(df),
    y
  ), drop = FALSE]

  Y <- df[[y]]

  if (ncol(X) < 2) {
    stop("Need at least two predictors")
  }

  idx <- stats::complete.cases(X, Y)

  X <- X[idx, , drop = FALSE]
  Y <- Y[idx]

  if (nrow(X) < 2) {
    stop("Not enough complete observations")
  }

  cm <- stats::cor(X)

  singular <- tryCatch(
    base::det(cm) < 1e-10,
    error = function(e) TRUE
  )

  bart <- NULL
  kmo <- NULL

  if (!singular) {

    bart <- tryCatch(
      psych::cortest.bartlett(
        cm,
        n = nrow(X)
      ),
      error = function(e) NULL
    )

    kmo <- tryCatch(
      psych::KMO(cm),
      error = function(e) NULL
    )
  }

  if (verbose) {

    message("\n--- PCA Suitability ---")

    if (!is.null(bart)) {

      message("\nBartlett Test")
      message(
        "Chi-square: ",
        round(bart$chisq, 3)
      )
      message("df: ", bart$df)
      message(
        "p-value: ",
        format.pval(bart$p.value)
      )

    } else {

      message("\nBartlett test not valid")
    }

    if (!is.null(kmo)) {

      message(
        "\nKMO: ",
        round(kmo$MSA, 3)
      )

    } else {

      message("\nKMO not valid")
    }
  }

  pca <- stats::prcomp(
    X,
    scale. = scale.,
    center = center
  )

  loadings <- as.data.frame(
    pca$rotation
  )

  scores <- as.data.frame(
    pca$x
  )

  variance <- pca$sdev^2 /
    sum(pca$sdev^2)

  if (verbose) {

    message("\n--- PCA Summary ---")

    print(summary(pca)$importance)

    message("\n--- PCA Loadings ---")

    print(round(loadings, 3))
  }

  ncomp <- which(
    cumsum(variance) >= 0.80
  )[1]

  ncomp <- min(
    ncomp,
    ncol(scores)
  )

  if (verbose) {

    message(
      "\nNumber of PCs retained: ",
      ncomp
    )
  }

  reg_data <- data.frame(
    Y = Y,
    scores[, 1:ncomp, drop = FALSE]
  )

  full_model <- stats::lm(
    Y ~ .,
    data = reg_data
  )

  backward_model <- tryCatch(
    stats::step(
      full_model,
      direction = "backward",
      trace = 0
    ),
    error = function(e) full_model
  )

  if (verbose) {

    message("\n--- Full PCR Model ---")

    print(summary(full_model))

    message(
      "\n--- Backward Selected Model ---"
    )

    print(summary(backward_model))
  }

  theme_pretty <-
    ggplot2::theme_minimal(
      base_family = "serif",
      base_size = 14
    ) +
    ggplot2::theme(
      plot.title =
        ggplot2::element_text(
          face = "bold",
          hjust = 0.5
        ),
      axis.title =
        ggplot2::element_text(
          face = "bold"
        ),
      axis.text =
        ggplot2::element_text(
          face = "bold",
          color = "black"
        ),
      axis.line =
        ggplot2::element_line(
          color = "black"
        ),
      axis.ticks =
        ggplot2::element_line(
          color = "black"
        ),
      panel.grid.minor =
        ggplot2::element_blank()
    )

  scree_data <- data.frame(
    PC = factor(
      paste0(
        "PC",
        seq_along(variance)
      ),
      levels = paste0(
        "PC",
        seq_along(variance)
      )
    ),
    Var = variance,
    CumVar = cumsum(variance)
  )

  scree_plot <- ggplot2::ggplot(
    scree_data,
    ggplot2::aes(x = PC)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(y = Var),
      fill = "#4C72B0",
      width = 0.6
    ) +
    ggplot2::geom_line(
      ggplot2::aes(
        y = CumVar,
        group = 1
      ),
      color = "#DD8452",
      linewidth = 1
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = CumVar),
      color = "#DD8452",
      size = 2.5
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        y = Var,
        label = paste0(
          round(Var * 100, 1),
          "%"
        )
      ),
      vjust = -0.4,
      size = 4,
      family = "serif",
      fontface = "bold"
    ) +
    ggplot2::scale_y_continuous(
      labels = scales::percent_format(),
      limits = c(0, 1)
    ) +
    ggplot2::labs(
      title = "PCA Scree Plot",
      x = "Principal Components",
      y = "Variance Explained"
    ) +
    theme_pretty

  biplot_obj <- NULL

  if (ncol(scores) >= 2) {

    load2 <- loadings[
      , 1:2,
      drop = FALSE
    ]

    load2$v <- rownames(load2)

    score2 <- scores[
      , 1:2,
      drop = FALSE
    ]

    scale_factor <- max(
      abs(score2)
    )

    biplot_obj <- ggplot2::ggplot() +

      ggplot2::geom_point(
        data = score2,
        ggplot2::aes(
          PC1,
          PC2
        ),
        color = "#2C7BB6",
        size = 3,
        alpha = 0.7
      ) +

      ggplot2::geom_segment(
        data = load2,
        ggplot2::aes(
          x = 0,
          y = 0,
          xend = PC1 * scale_factor,
          yend = PC2 * scale_factor
        ),
        arrow = grid::arrow(
          length = grid::unit(
            0.25,
            "cm"
          )
        ),
        color = "#D7191C",
        linewidth = 1
      ) +

      ggrepel::geom_text_repel(
        data = load2,
        ggplot2::aes(
          PC1 * scale_factor,
          PC2 * scale_factor,
          label = v
        ),
        family = "serif",
        fontface = "bold",
        color = "#D7191C",
        size = 4
      ) +

      ggplot2::labs(
        title = "PCA Biplot",
        x = "PC1",
        y = "PC2"
      ) +

      ggplot2::coord_equal() +
      theme_pretty

    gridExtra::grid.arrange(
      scree_plot,
      biplot_obj,
      ncol = 2
    )

  } else {

    warning(
      paste(
        "Biplot requires at least",
        "two principal components"
      )
    )

    print(scree_plot)
  }

  invisible(list(
    pca = pca,
    bartlett = bart,
    kmo = kmo,
    loadings = loadings,
    scores = scores,
    variance = variance,
    ncomp = ncomp,

    full_model = full_model,
    full_model_summary =
      summary(full_model),

    backward_model = backward_model,
    backward_model_summary =
      summary(backward_model),

    scree_plot = scree_plot,
    biplot = biplot_obj
  ))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/supervised_pca.R
\name{supervised_pca}
\alias{supervised_pca}
\title{Supervised Principal Component Analysis (SPCA)}
\usage{
supervised_pca(data, response, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{Numeric data frame or matrix}

\item{response}{Numeric response vector}

\item{K}{Number of components (auto if NULL)}

\item{threshold}{Variance threshold for K selection}

\item{verbose}{Logical; if TRUE prints progress messages}
}
\value{
List of SPCA results
}
\description{
True SPCA (Bair et al.): screening + PCA
}
\examples{
data(mtcars)
Result <- supervised_pca(
  mtcars[, -1],
  mtcars$mpg
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robust_pcreg.R
\name{robust_pcreg}
\alias{robust_pcreg}
\title{Robust Principal Component Regression (Robust PCR)}
\usage{
robust_pcreg(data, y, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{A data frame or matrix}

\item{y}{Response variable name (character)}

\item{K}{Number of components (optional)}

\item{threshold}{Variance threshold for automatic K selection}

\item{verbose}{Logical; print detailed output}
}
\value{
A list containing Robust PCR outputs
}
\description{
Performs Robust PCA using Minimum Covariance Determinant (MCD)
followed by regression on robust principal components.
Includes PCA suitability tests, summary, loadings,
scree plot, biplot, and regression models.
}
\examples{
data(mtcars)
Result <- robust_pcreg(mtcars, y = "mpg")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robust_pca.R
\name{robust_pca}
\alias{robust_pca}
\title{Robust Principal Component Analysis (Robust PCA)}
\usage{
robust_pca(data, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{Numeric data frame or matrix}

\item{K}{Number of components (auto if NULL)}

\item{threshold}{Variance threshold for K selection}

\item{verbose}{Logical; if TRUE prints progress messages}
}
\value{
List of results
}
\description{
Performs Robust PCA using MCD estimator with automatic K selection,
PCA suitability tests (Bartlett & KMO), scree plot and biplot.
}
\examples{
data(iris)
Result <- robust_pca(iris[, 1:4])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{pca}
\alias{pca}
\title{Principal Component Analysis (Custom Scree Style)}
\usage{
pca(data, scale. = TRUE, center = TRUE, verbose = FALSE)
}
\arguments{
\item{data}{A data frame or matrix}

\item{scale.}{Logical; scale variables}

\item{center}{Logical; center variables}

\item{verbose}{Logical; if TRUE, prints Bartlett and KMO results.}
}
\value{
List with Bartlett test, KMO, loadings, scores, variance
}
\description{
Performs PCA with Bartlett test, KMO measure,
and visualizes scree plot and biplot (custom style).
}
\examples{
{
data(mtcars)
Result <- pca(mtcars)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparse_pcreg.R
\name{sparse_pcreg}
\alias{sparse_pcreg}
\title{Sparse Principal Component Regression (Sparse PCR)}
\usage{
sparse_pcreg(data, y, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{A data frame or matrix}

\item{y}{Response variable name (character)}

\item{K}{Number of components (optional)}

\item{threshold}{Variance threshold for automatic K selection}

\item{verbose}{Logical; print messages and summaries}
}
\value{
A list containing model outputs and PCA results
}
\description{
Performs Sparse PCA followed by regression with automatic selection
of number of components. Includes PCA suitability tests, loadings,
regression models, scree plot, and biplot.
}
\examples{
data(mtcars)
Result <- sparse_pcreg(mtcars, y = "mpg")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pc_reg.R
\name{pcreg}
\alias{pcreg}
\title{Principal Component Regression (PCR)}
\usage{
pcreg(data, y, scale. = TRUE, center = TRUE, verbose = FALSE)
}
\arguments{
\item{data}{A data frame or matrix}

\item{y}{Response variable name}

\item{scale.}{Logical; should variables be scaled?}

\item{center}{Logical; should variables be centered?}

\item{verbose}{Logical; print summaries?}
}
\value{
A list containing PCA results,
regression models, diagnostics, and plots.
}
\description{
Performs Principal Component Analysis (PCA) on predictors
followed by regression using selected principal components.
Includes Bartlett's test, KMO measure,
scree plot, biplot, and backward model selection.
}
\examples{
data(mtcars)
result <- pcreg(mtcars, y = "mpg")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparse_pca.R
\name{sparse_pca}
\alias{sparse_pca}
\title{Sparse Principal Component Analysis (Sparse PCA)}
\usage{
sparse_pca(data, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{Numeric data frame or matrix}

\item{K}{Number of components (auto if NULL)}

\item{threshold}{Variance threshold for automatic K selection}

\item{verbose}{Logical; prints diagnostic messages}
}
\value{
List with PCA results
}
\description{
Performs Sparse PCA using elastic net regularisation with custom scree plot and biplot.
}
\examples{
{
data(mtcars)
Result <- sparse_pca(mtcars)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/supervised_pcr.R
\name{supervised_pcreg}
\alias{supervised_pcreg}
\title{Supervised Principal Component Regression}
\usage{
supervised_pcreg(data, response, K = NULL, threshold = 0.9, verbose = FALSE)
}
\arguments{
\item{data}{A data frame or matrix}

\item{response}{Response variable name}

\item{K}{Number of principal components}

\item{threshold}{Cumulative variance threshold
for automatic component selection}

\item{verbose}{Logical; print summaries?}
}
\value{
A list containing PCA results,
regression models, diagnostics, and plots.
}
\description{
Performs supervised feature selection followed by
Principal Component Regression (PCR).
Includes Bartlett's test, KMO measure,
scree plot, biplot, and regression models.
}
\examples{
data(mtcars)
result <- supervised_pcreg(
  mtcars,
  response = "mpg"
)
}
